function [HL LM]=auto_LM_face(V,F,Curvature,Meanshape)

%This algorithm generates a preliminary set of control points on a human face that is subsequently used to guide a dense deformation of a template with landmarks to the subject. And then it transfers the landmarks from the template to the subject in an automated manner.
%The algorithm was designed specifically for a data set of children from Tanzania. 


%Input: V is N-by-D matrix, where N is the number of vertices of one
%       sample, D is the dimension of the vertices. For 3D mesh data, D=3.
%
%       F is M-by-3 matrix, where M is the number of faces
%
%       Curvature is the curvature information on the vertices computed
%       through mesh. In this code, it is N-by-3 matrix represents the color
%       information of the curvature. One way to compute it is to use
%       Meshlab software.
%
%       Meanshape.V is vertices; 
%       Meanshape.F is faces;
%       Meanshape.HL(1:17,:) is the control points
%       Meanshape.HL(18:end,:) is the landmarks.

%Output: HL is 17-by-3 matrix represents the control points
%
%        LM is 29-by-3 matrix represents the landmarks



%The number of row of V should be same as the number of Curvature
if size(V,1)~=size(Curvature,1)
    error('The number of row of V should be same as the number of Curvature.');
end

%if curvature is represented by color, then we will index it
if size(Curvature,2)==3
    B=jet(1024);
    D = pdist2(Curvature,B);
    Ind=[];
    for i=1:length(Curvature)
        [m1 m2]=min(D(i,:));
        Ind=[Ind m2];
    end
    clear B D
else
    error('Please check the Curvature information.');
end

%reference points
R=[-13.4729   28.1495   -1.6769
   13.0000   29.2936   -2.8392
   -2.5366   27.8519    3.1870
   -0.2630   -2.7848   14.7345];

%compute the control points
HL=Meanshape.HL(1:17,:);
HL=mao_hflandmark(V,Ind,R);

%transfer by thin plate spline
LM=transfer_LM(V,F,HL,Meanshape);


function DD = pdist2(A,B,Wts)
% Find pair-wise SQUARE EUCLIDEAN distance
% or 'Weighted square euclidean' distance
% between each point in A and B
% For 2 vector a, b
% Euclidean distance= d = sum((a-b).^2)
% Weighted version  = d = sum(wts.*(a-b).^2)
% ------------------------------
% Input:
%   A= m_by_p, m points in p-dimension
%   B= n_by_p, n points in p-dimension
%   Wts = 1_by_p, defaut = [1 1 ...]
% Results:
%   DD= m_by_n
% ------------------------------
% Facts: (a-b)^2= a^2+b^2-2ab
% ------------------------------
% trungd@okstate.edu, Feb 2011
% ------------------------------
% Ideal from: Piotr Dollar.  [pdollar-at-caltech.edu]

[m,p1] = size(A); [n,p2] = size(B);
if(p1~=p2) % check size
    error('Must have: ncol(X)=ncol(Y)=length(Wts)')
end
if nargin < 3 % standard euclidean
    AA = sum(A.*A,2);  % column m_by_1
    BB = sum(B.*B,2)'; % row 1_by_n
    DD = AA(:,ones(1,n)) + BB(ones(1,m),:) - 2*A*B';
else
    if p1 ~=length(Wts)
        error('Must have: ncol(X)=ncol(Y)=length(Wts)')
    end
    sW = sqrt(Wts(:)'); % make sure row, square of Wts
    A = sW(ones(1,m),:).*A;
    B = sW(ones(1,n),:).*B; % modify A,B
    % Process the same as standard Euclidean
    AA = sum(A.*A,2);  % column m_by_1
    BB = sum(B.*B,2)'; % row 1_by_n
    DD = AA(:,ones(1,n)) + BB(ones(1,m),:) - 2*A*B';    
end

function HL=mao_hflandmark(V,Ind,R)
% compute the control points

%scale the face
% Scale1=mean(V);
% Scale2=max(V(:,2))-min(V(:,2));
% V=V-repmat(Scale1,length(V),1);
% V=V/Scale2*170;

%find most positive and most negative curvature parts
A=find(Ind>0.59*max(Ind)+0.41*min(Ind));
if length(A)>length(V)*0.7
    A=find(Ind>0.67*max(Ind)+0.33*min(Ind));
end
if length(A)>length(V)*0.17
    A=find(Ind>0.61*max(Ind)+0.39*min(Ind));
end
if  length(A)<length(V)*0.03
    A=find(Ind>0.50*max(Ind)+0.50*min(Ind));
end
if length(A)<length(V)*0.08
    A=find(Ind>0.56*max(Ind)+0.44*min(Ind));
end
B=find(Ind<0.4*max(Ind)+0.6*min(Ind));
if length(B)>0.2*length(V)
    B=find(Ind<0.35*max(Ind)+0.65*min(Ind));
end
if length(B)<0.04*length(V)
    B=find(Ind<0.5*max(Ind)+0.5*min(Ind));
end
 if length(B)<0.03*length(V)
 B=find(Ind<0.57*max(Ind)+0.43*min(Ind));
  end


m=mean(V(A,:));
m(:,1)=(max(V(:,1))+min(V(:,1)))/2;
m(:,2)=(max(V(:,2))+min(V(:,2)))/2;
DD=pdist2(V(A,1:2),m(:,1:2));
a=find(DD<2300&V(A,3)>m(3));
chin=mean(V(A(a),:));
V=V-repmat(chin,length(V),1);
%denoise
e=mao_kickpt(V(A,:));
A(e)=[];
U=V(A,:);
e=mao_kickpt(V(B,:));
B(e)=[];
U2=V(B,:);
mid=find(abs(U2(:,2))<40);
[m1 m2]=max(U2(mid,1));
[m3 m4]=min(U2(mid,1));
theta=(U2(mid(m4),3)-U2(mid(m2),3))/norm(U2(mid(m4),:)-U2(mid(m2),:));
Rot=[cos(theta) 0 sin(theta);0 1 0;-sin(theta) 0 cos(theta)];
Rotcent=(U2(m2,:)+U2(m4,:))/2;
V=(V-repmat(Rotcent,length(V),1))*Rot+repmat(Rotcent,length(V),1);
U=V(A,:);
U2=V(B,:);

m=mean(U);
m(:,1)=(max(V(:,1))+min(V(:,1)))/2;
m(:,2)=(max(V(:,2))+min(V(:,2)))/2;
DD=pdist2(U(:,1:2),m(:,1:2));
a=find(DD<2000&U(:,3)>m(3));
chin6=mean(U(a,:));
V=V-repmat(chin6,length(V),1);
U=V(A,:);
U2=V(B,:);

%y axis rotate
[COEFF,SCORE] = princomp(U(a,:));
theta2=COEFF(2,3)/COEFF(3,3)-0.175;
Rot2=[1 0 0;0 cos(theta2) sin(theta2);0 -sin(theta2) cos(theta2)];
V=V*Rot2;
U=V(A,:);
U2=V(B,:);

m=mean(U);
m(:,1)=(max(V(:,1))+min(V(:,1)))/2;
m(:,2)=(max(V(:,2))+min(V(:,2)))/2;
DD=pdist2(U(:,1:2),m(:,1:2));
a=find(DD<2000&U(:,3)>m(3));
chin7=mean(U(a,:));
V=V-repmat(chin7,length(V),1);
U=V(A,:);
U2=V(B,:);

a=find(U(:,2)<50&U(:,2)>-30);
[COEFF,SCORE] = princomp(U(a,1:2));
[t1 t2]=max(SCORE(:,1));
HL(16,:)=U(a(t2),:);
[t1 t2]=min(SCORE(:,1));
HL(17,:)=U(a(t2),:);
if abs(HL(16,2)-HL(17,2))>25
    sm=min(HL(16,2),HL(17,2));
    HL(16,2)=sm;
    HL(17,2)=sm;
end


D1 = pdist2(U,R(1,:));
[p1 p2]=min(D1);
D1=pdist2(U,U(p2,:));
a=find(D1<180);
if ~isempty(a)
E1=U(a,:);
else
    fprintf('Warning:HL1 can not be computed\n');
    return
    end
D1 = pdist2(U,R(2,:));
[p1 p2]=min(D1);
D1=pdist2(U,U(p2,:));
a=find(D1<180);
if ~isempty(a)
E2=U(a,:);
else
    fprintf('Warning:HL1 can not be computed\n');
    return
    end
ins=intersect(E1,E2,'rows');
if isempty(ins)
DD=pdist2(E1,E2);
[m1,m2]=find(DD==min(min(DD)));
HL(1,:)=E1(m1,:); %right eye
HL(2,:)=E2(m2,:); %left eye
else
    D11=pdist2(setdiff(E1,ins,'rows'),ins(1,:));
    D12=pdist2(setdiff(E2,ins,'rows'),ins(1,:));
    if min(D11)<min(D12)
        E2=setdiff(E2,ins,'rows');
    else
        E1=setdiff(E1,ins,'rows');
    end
    DD=pdist2(E1,E2);
    if ~isempty(DD)
[m1,m2]=find(DD==min(min(DD)));
HL(1,:)=E1(m1,:); %right eye
HL(2,:)=E2(m2,:); %left eye
else
    fprintf('Warning:HL1 can not be computed\n');
    return
    end
end

if HL(1,2)-HL(2,2)>7.5
    a=find(U(:,1)>HL(1,1)+15&U(:,1)<HL(1,1)+35&abs(U(:,2)-HL(1,2))<5);
    if ~isempty(a)
    E2=U(a,:);
    DD=pdist2(E1,E2);
    [m1,m2]=find(DD==min(min(DD)));
    HL(2,:)=E2(m2,:);
    end
elseif HL(2,2)-HL(1,2)>7.5
    a=find(U(:,1)<HL(2,1)-15&U(:,1)>HL(2,1)-35&abs(U(:,2)-HL(2,2))<5);
    if ~isempty(a)
    E1=U(a,:);
    DD=pdist2(E1,E2);
    [m1,m2]=find(DD==min(min(DD)));
    HL(1,:)=E1(m1,:);
    end
end

if abs(HL(1,1)-HL(2,1))<17.5
    if abs(HL(1,1))<abs(HL(2,1))
        a=find(U(:,1)<HL(1,1)-5&U(:,1)>HL(1,1)-20&abs(U(:,2)-HL(2,2))<5);
        if ~isempty(a)
            E1=U(a,:);
            DD=pdist2(E1,E2);
            [m1,m2]=find(DD==min(min(DD)));
            HL(1,:)=E1(m1,:);
        end
    else
        a=find(U(:,1)<HL(2,1)+20&U(:,1)>HL(2,1)+5&abs(U(:,2)-HL(1,2))<5);
        if ~isempty(a)
            E2=U(a,:);
            DD=pdist2(E1,E2);
            [m1,m2]=find(DD==min(min(DD)));
            HL(2,:)=E2(m2,:);
        end
    end
end

if abs(HL(1,1)-HL(2,1))>41
    D1 = pdist2(U,HL(1,:));
[p1 p2]=min(D1);
D1=pdist2(U,U(p2,:));
a=find(D1<300);
E1=U(a,:);
D1 = pdist2(U,HL(2,:));
[p1 p2]=min(D1);
D1=pdist2(U,U(p2,:));
a=find(D1<300);
E2=U(a,:);
DD=pdist2(E1,E2);
            [m1,m2]=find(DD==min(min(DD)));
            HL(1,:)=E1(m1,:);
            HL(2,:)=E2(m2,:);
end
    
    
    
        
    
DD = pdist2(V,HL(1:2,:));
a=find(abs(DD(:,1)-DD(:,2))<100);
[m1 m2]=min(DD(a,1));
[m3 m4]=min(DD(a,2));
Mid=(V(a(m2),:)+V(a(m4),:))/2;
DD = pdist2(V,Mid);
[m1 m2]=min(DD);
HL(3,:)=V(m2,:);%between eyes

D3 = pdist2(V(a,:),HL(3,:));
b=find(D3<850&D3>600);
M=V(a(b),:);
[m1 m2]=min(pdist2(M,R(4,:)));
M=M(m2,:);
D3 = pdist2(V(a,:),M);
b=find(D3<500);
if ~isempty(b)
[COEFF,SCORE] = princomp(V(a(b),:));
[m1 m2]=min(SCORE(:,2));
[m3 m4]=max(SCORE(:,2));
if abs(V(a(b(m2)),3))<abs(V(a(b(m4)),3))
    m2=m4;
end
HL(4,:)=V(a(b(m2)),:);%nose
else
    fprintf('Warning:HL4 can not be computed\n');
    return
    end

M=HL(4,:);
M(:,1)=(HL(4,1)+HL(3,1))/2;
D4 = pdist2(U,M);
b=find(D4<600);
[COEFF,SCORE] = princomp([U(b,1:2);M(:,1:2)]);
c=find(SCORE(:,1)>-10+SCORE(length(SCORE),1)&SCORE(:,1)<10+SCORE(length(SCORE),1));
if length(find(SCORE(c,2)>SCORE(length(SCORE),2)))>length(find(SCORE(c,2)<SCORE(length(SCORE),2)))
    c=find(SCORE(:,1)<10+SCORE(length(SCORE),1)&SCORE(:,1)>SCORE(length(SCORE),1)&SCORE(:,2)>SCORE(length(SCORE),2)+3);
    if ~isempty(c)
    [m1 m2]=min(SCORE(c,2));
    HL(6,:)=U(b(c(m2)),:);%left nose
    else
    fprintf('Warning:HL6 can not be computed\n');
    return
    end
    c=find(SCORE(:,1)>-10+SCORE(length(SCORE),1)&SCORE(:,1)<SCORE(length(SCORE),1)&SCORE(:,2)>SCORE(length(SCORE),2)+3);
    if ~isempty(c)
    [m1 m2]=min(SCORE(c,2));
    HL(7,:)=U(b(c(m2)),:);%right nose
    else
    fprintf('Warning:HL7 can not be compute\n');
    return
    end
else
    c=find(SCORE(:,1)<10+SCORE(length(SCORE),1)&SCORE(:,1)>SCORE(length(SCORE),1)&SCORE(:,2)<SCORE(length(SCORE),2)-3);
    if ~isempty(c)
    [m1 m2]=max(SCORE(c,2));
    HL(6,:)=U(b(c(m2)),:);%left nose
    else
    fprintf('Warning:HL6 can not be computed\n');
    return
    end
    c=find(SCORE(:,1)>-10+SCORE(length(SCORE),1)&SCORE(:,1)<SCORE(length(SCORE),1)&SCORE(:,2)<SCORE(length(SCORE),2)-3);
    if ~isempty(c)
    [m1 m2]=max(SCORE(c,2));
    HL(7,:)=U(b(c(m2)),:);%right nose   
    else
    fprintf('Warning:HL6 can not be computed\n');
    return
    end
end
if HL(6,1)<HL(7,1)
    tmp=HL(6,:);
    HL(6,:)=HL(7,:);
    HL(7,:)=tmp;
end 

if abs(HL(6,2)-HL(7,2))>4
    if HL(7,2)>HL(6,2)
        g=find(U(:,2)>HL(6,2)&U(:,2)<HL(7,2)&U(:,1)>HL(4,1)+1&U(:,1)<2*HL(4,1)-HL(7,1));
        if ~isempty(g)
        [m1 m2]=max(U(g,2));
        HL(6,:)=U(g(m2),:);
        end
    else
         g=find(U(:,2)>HL(7,2)&U(:,2)<HL(6,2)&U(:,1)<HL(4,1)-1&U(:,1)>2*HL(4,1)-HL(7,1));
         if ~isempty(g)
        [m1 m2]=max(U(g,2));
        HL(7,:)=U(g(m2),:);
        end
    end
end

if HL(6,1)<HL(7,1)
    tmp=HL(6,:);
    HL(6,:)=HL(7,:);
    HL(7,:)=tmp;
end 

if abs(HL(6,1)-HL(7,1))<6
    if HL(6,1)<HL(4,1)+2
       DD=pdist2(U,HL(6,:)+[15 2 0]);
       [m1 m2]=min(DD);
       HL(6,:)=U(m2,:);
    end
    if HL(7,1)>HL(4,1)-2
        DD=pdist2(U,HL(7,:)+[-15 2 0]);
       [m1 m2]=min(DD);
       HL(7,:)=U(m2,:);
    end
end

D5 = pdist2(U(b,:),HL(6:7,:));
c=find(abs(D5(:,1)-D5(:,2))<50&U(b,2)<HL(4,2));
if ~isempty(c)
    if length(c)>1
        M=mean(U(b(c),:));
    else M=U(b(c),:);
    end
    [m1 m2]=min(pdist2(U(b,:),M));
    HL(5,:)=U(b(m2),:);%bottom nose
else
    D5 = pdist2(V,HL(6:7,:));
    c=find(abs(D5(:,1)-D5(:,2))<50&V(:,2)<HL(4,2)&V(:,2)>HL(4,2)-20);
    if ~isempty(c)
        M=V(c,:);
        [m1 m2]=min(M(:,3));
        HL(5,:)=M(m2(1),:);
    else
    fprintf('Warning:HL5 can not be computed\n');
    return
    end
end


Mouse1=HL(7,:)+(HL(7,:)-HL(2,:))/1;
Mouse2=HL(5,:)+(HL(5,:)-HL(3,:))/2.5;
Mouse3=HL(6,:)+(HL(6,:)-HL(1,:))/1;
% DD=pdist2(U(:,1:2),Mouse1(:,1:2));
% [m1 m2]=min(DD);
% Mouse1=U(m2,:);
% DD=pdist2(U(:,1:2),Mouse3(:,1:2));
% [m1 m2]=min(DD);
% Mouse3=U(m2,:);

DD=pdist2(U,Mouse1);
[m1 m2]=min(DD);
Mouse1=U(m2,:);
DD=pdist2(U,Mouse3);
[m1 m2]=min(DD);
Mouse3=U(m2,:);

if abs(Mouse1(:,1)-Mouse3(:,1))<25
    Mouse1(:,1)=HL(1,1)-10;
    Mouse3(:,1)=HL(2,1)+10;
end
if abs(Mouse1(:,2)-Mouse3(:,2))>5
    if abs(Mouse1(:,2)-Mouse2(:,2))<abs(Mouse3(:,2)-Mouse2(:,2))
        Mouse3(:,1)=HL(2,1)+10;
        Mouse3(:,2)=Mouse1(:,2);
    else
        Mouse1(:,1)=HL(1,1)-10;
        Mouse1(:,2)=Mouse3(:,2);
    end
end

b1=find(U(:,1)<Mouse1(:,1)+15&U(:,1)>Mouse1(:,1)-10&abs(U(:,2)-Mouse1(:,2))<5&U(:,3)>Mouse1(:,3)-10);
b2=find(abs(U(:,1)-Mouse2(:,1))<10&abs(U(:,2)-Mouse2(:,2))<4);
b3=find(U(:,1)<Mouse3(:,1)+10&U(:,1)>Mouse3(:,1)-15&abs(U(:,2)-Mouse3(:,2))<5&U(:,3)>Mouse3(:,3)-10);

B=[b1;b2;b3];
if ~isempty(B)
[COEFF,SCORE] = princomp(U(B,:));
[m1 m2]=max(SCORE(:,1));
HL(10,:)=U(B(m2),:);%left mouth
[m1 m2]=min(SCORE(:,1));
HL(11,:)=U(B(m2),:);%right mouth
if norm(HL(10,:)-HL(1,:))<norm(HL(11,:)-HL(1,:))
    tmp=HL(11,:);
    HL(11,:)=HL(10,:);
    HL(10,:)=tmp;
end
else
    fprintf('Warning:HL10 can not be computed\n');
    return
end

DD = pdist2(V,HL(10:11,:));
a=find(abs(DD(:,1)-DD(:,2))<100);
D3 = pdist2(V(a,:),(HL(10,:)+HL(11,:))/2);
b=find(D3<250);
DDD=pdist2(V(a(b),:),U(B,:));
[p1 p2]=find(DDD==min(min(DDD)));
if ~isempty(p1)
HL(14,:)=V(a(b(p1(1))),:);%middle mouth
else
    fprintf('Warning:HL14 can not be computed\n');
    return
end

D3 = pdist2(V(a,:),HL(14,:));
b=find(D3<300);
M=[V(a(b),:);HL(14,:)];
[COEFF,SCORE] = princomp(M);
DD=pdist2(SCORE,SCORE(length(SCORE),:));
p=find(DD<60);
if length(find(SCORE(p,2)>SCORE(length(SCORE),2)))>length(find(SCORE(p,2)<SCORE(length(SCORE),2)))
c=find(SCORE(:,1)>SCORE(length(SCORE),1));
[m1 m2]=max(SCORE(c,2));
HL(12,:)=V(a(b(c(m2))),:);%top or bottom mouth
c=find(SCORE(:,1)<SCORE(length(SCORE),1));
[m1 m2]=max(SCORE(c,2));
HL(13,:)=V(a(b(c(m2))),:);%top or bottom mouth
[m1 m2]=min(SCORE(p,2));
if p(m2)<=length(b)
HL(14,:)=V(a(b(p(m2))),:);
end
else
c=find(SCORE(:,1)>SCORE(length(SCORE),1));
[m1 m2]=min(SCORE(c,2));
HL(12,:)=V(a(b(c(m2))),:);%top or bottom mouth
c=find(SCORE(:,1)<SCORE(length(SCORE),1));
[m1 m2]=min(SCORE(c,2));
HL(13,:)=V(a(b(c(m2))),:);%top or bottom mouth
[m1 m2]=max(SCORE(p,2));
if p(m2)<=length(b)
HL(14,:)=V(a(b(p(m2))),:);
end
end



if norm(HL(12,:)-HL(3,:))>norm(HL(13,:)-HL(3,:))
    tmp=HL(13,:);
    HL(13,:)=HL(12,:);
    HL(12,:)=tmp;
end

D3 = pdist2(V(a,:),HL(13,:));
b=find(D3<300);
M=V(a(b),:);
M=[M;HL(13:14,:)];
[COEFF,SCORE] = princomp(M);
k=length(SCORE);
if SCORE(k-1,1)>SCORE(k,1)
     c=find(SCORE(:,1)>SCORE(k-1));
     if ~isempty(c)
    if SCORE(k-1,2)>SCORE(k,2)
        [m1 m2]=min(SCORE(c,2));
    else
        [m1 m2]=max(SCORE(c,2));
    end
    else
    fprintf('Warning:HL15 can not be computed\n');
    return
end
else
    c=find(SCORE(:,1)<SCORE(k-1));
    if ~isempty(c)
    if SCORE(k-1,2)>SCORE(k,2)
        [m1 m2]=min(SCORE(c,2));
    else
        [m1 m2]=max(SCORE(c,2));
    end
    else
    fprintf('Warning:HL15 can not be computed\n');
    return
end
end
HL(15,:)=M(c(m2),:);%chin

D5= pdist2(U2,HL(16,:));
b=find(D5<2500);
[p1 p2]=min(pdist2(U2(b,:),(HL(15,:)-[0 0 60])));
if ~isempty(p2)
HL(16,:)=U2(b(p2),:);%right ear
else
    fprintf('Warning:HL16 can not be compute\n');
    return
end

D5= pdist2(U2,HL(17,:));
b=find(D5<2500);
[p1 p2]=min(pdist2(U2(b,:),(HL(15,:)-[0 0 60])));
if ~isempty(p2)
HL(17,:)=U2(b(p2),:);%left ear
else
    fprintf('Warning:HL17 can not be computed\n');
    return
end


D5= pdist2(U2,HL(6,:));
b1=find(D5<100);
D5= pdist2(U2,HL(7,:));
b2=find(D5<100);
D5= pdist2(U2,HL(4,:));
b3=find(D5<100);
b=[b1;b2;b3];

[COEFF,SCORE] = princomp(U2(b,:));
c1=find(SCORE(:,1)>max(SCORE(:,1))*2/3);
c2=find(SCORE(:,1)<min(SCORE(:,1))*2/3);
if (~isempty(c1))&&(~isempty(c2))
[m m1]=max(SCORE(c1,2));
[m m2]=max(SCORE(c2,2));
[m m3]=min(SCORE(c1,2));
[m m4]=min(SCORE(c2,2));
tmp(1,:)=U2(b(c1(m1)),:);
tmp(2,:)=U2(b(c2(m2)),:);
tmp(3,:)=U2(b(c1(m3)),:);
tmp(4,:)=U2(b(c2(m4)),:);
[m r]=min(pdist2(tmp,HL(10,:)));
HL(8,:)=tmp(r,:);%left nose root
[m r]=min(pdist2(tmp,HL(11,:)));
HL(9,:)=tmp(r,:);%left nose root
else
    fprintf('Warning:HL8&9 can not be computed\n');
    return
end

if HL(16,1)>HL(17,1)
    tmp=HL(17,:);
    HL(17,:)=HL(16,:);
    HL(16,:)=tmp;
end
if HL(11,1)>HL(10,1)
    tmp=HL(10,:);
    HL(10,:)=HL(11,:);
    HL(11,:)=tmp;
end
if HL(9,1)>HL(8,1)
    tmp=HL(9,:);
    HL(9,:)=HL(8,:);
    HL(8,:)=tmp;
end

b=[3 4 5 12 14 13 15];
XYZ=[HL(b,:)];
alpha=polyfit(XYZ(:,2),XYZ(:,1),1);
a1=alpha(1);b1=alpha(2);
X=XYZ(:,2);Y=XYZ(:,1);
clear Q
Q(:,2)=(X+a1*Y-a1*b1*ones(size(Y)))./(a1^2+1);
Q(:,1)=Q(:,2)*a1+b1*ones(size(Y));
HL(b,1:2)=Q;

HL=HL+repmat(chin7,length(HL),1);
HL=HL*Rot2';
HL=HL+repmat(chin6,length(HL),1);
HL=(HL-repmat(Rotcent,length(HL),1))*Rot'+repmat(Rotcent,length(HL),1);
HL=HL+repmat(chin,length(HL),1);
% HL=HL/170*Scale2;
% HL=HL+repmat(Scale1,length(HL),1);

function e=mao_kickpt(X)
%denoise the sparse points
if size(X,1)<size(X,2)
    X=X';
end

D=pdist(X);
W=squareform(D);
e=[];
for j=1:length(W)
    a=find(W(j,:)<6);
    if length(a)<20
        e=[e j];
    end
end
X(e,:)=[];
  
function LM=transfer_LM(V,F,HL,Meanshape)
k=1;
tot_err=1e-4;
distance(k)=inf;
lambda=0.01;

p=HL;
p_mean=Meanshape.HL(1:17,:);
[p_mean centerm scalem]=pre_shape(p_mean);

[trash center scale]=pre_shape(p);
[trash1 trash2 trash3 r]=finddistance(p_mean,p);

Tri0=Meanshape.F;
score0=Meanshape.V;
Tri0=Tri0';
score0=score0';
ctrl0_template=Meanshape.HL(18:end,:);
score0=(((score0'-repmat(centerm,size(score0,2),1))./scalem)')';
ctrl0_template=(((ctrl0_template-repmat(centerm,size(ctrl0_template,1),1))./scalem)')';
I0_template=nearestneighbour(ctrl0_template',score0');
ctrl0=p_mean;
I0=nearestneighbour(ctrl0',score0');
I00=I0;

num_new=17;
Tri1=F';
Pts1=V';
Pts1=(r*((Pts1'-repmat(center,size(Pts1,2),1))./scale)')';
ctrl1=(r*((p-repmat(center,size(p,1),1))./scale)')';

% ** try the normal spline ** %
[K] = tps_compute_dist_matrix(ctrl0);
[Pm,Q1,Q2,R] = tps_set_matrices(ctrl0);
[affine non_affine] = tps_compute_params(Q1,Q2,R,ctrl1,K,lambda);
all_parameter=[reshape(affine,16,1);reshape(non_affine,size(non_affine,1)*size(non_affine,2),1)];
all_paramter1_temp=[affine;non_affine];
all_paramter1=reshape(all_paramter1_temp(:,2:end),size(all_paramter1_temp(:,2:end),1)*size(all_paramter1_temp(:,2:end),2),1);
new_pt_temp=tps_defomed_pts(score0,affine ,non_affine,ctrl0);
I_temp=nearestneighbour(new_pt_temp(:,2:end)',Pts1');
new_pt=Pts1(I_temp,:);
ctrl_new1=((r\new_pt(I0_template,:)')'.*scale)+repmat(center,size(I0_template,2),1);

b=[1:8 29];
ctrl_new2=ctrl_new1;
XYZ=[ctrl_new1(b,:)];
alpha=polyfit(XYZ(:,2),XYZ(:,1),1);
a1=alpha(1);b1=alpha(2);
X=ctrl_new1(b,2);Y=ctrl_new1(b,1);
clear Q
Q(:,2)=(X+a1*Y-a1*b1*ones(size(Y)))./(a1^2+1);
Q(:,1)=Q(:,2)*a1+b1*ones(size(Y));
ctrl_new2(b,1:2)=Q;

LM=ctrl_new2;

% Find the Pre_shape for any arbitary shape
function [p c scale]=pre_shape(s)
% s is the shape sample in the n*k, n points and each point lies in R^k
c=mean(s);
p=s-repmat(c,size(s,1),1);% centerlize the data.
scale=norm(p,'fro');
p=p/scale;


% find distance for two shapes
function [distance F2 temp tran]= finddistance(dataN,dataN1)
% dataN and dataN1 N*k  two preshapes
% N is the number of points, and k is the dimension


%output
%distance is the geodestic distance between dataN and dataN1;
%F2 is the Uipi
%temp is the zetai
%tran is the rotation matrix
[dataN cN scaleN]=pre_shape(dataN);
[dataN1 cN1 scaleN1]=pre_shape(dataN1);
C=dataN'*dataN1;
[S,V,D]=svd(C);
tran=S*transpose(D);
F=S*transpose(D)*dataN1';% the final minimizer
F=F';
% E=dataN;
% E2=E/norm(E,'fro');
F2=F/norm(F,'fro');
temp=sum(diag(V));
distance = acos(temp);
function [K] = tps_compute_dist_matrix(x)
% compute the kernel function based on the control points

% Input X in N*k (k is the dimension)
% Output
% K is the kernel matrix
% 1D :K = r^3
% 2D: K = r^2 * log r
% 3D: K = -r
% ave is the mean distance between cotrol points
[n, d] = size (x);
K = zeros (n,n);

for i=1:d
    tmp = x(:,i) * ones(1,n) - ones(n,1) * x(:,i)';
    tmp = tmp .* tmp;
    K = K + tmp;
end;

if d==1
    K=sqrt(K).^3;
else
    if d == 2
        mask = K < 1e-10; % to avoid singularity.
        K = 0.5 * K .* log(K + mask) .* (K>1e-10);
    else
        K = - sqrt(K);
    end;
end

function [affine non_affine]=tps_compute_params(Q1,Q2,R,Y1,K,lambda,varargin)
% Y is the tagent points n*2 or n*3
% param=[affine:non_affine]
% input a column vector as a weight vector for those points
Y=tps_n_c(Y1);
n=size(K,1);
if isempty(varargin)
    w=diag(ones(n,1));
else
    if length(varargin)==1
    w=diag(varargin{1});
    else
        error('only accept one more paramaters');
    end
end
if size(w)~=[n,n]
    error('weight vector need to be same length as the # of points');
    
end
M=K+n*lambda*inv(w^2);
non_affine=(Q2/(Q2'*M*Q2))*Q2'*Y;
affine=(R\Q1')*(Y-M*non_affine);

function [new_pt K]=tps_defomed_pts(pt,affine,non_affine,ctrl_pts)
[m1,d]=size(ctrl_pts);
%K=pdist2(pt,ctrl_pts,'euclidean');
Yt = ctrl_pts';
XX = sum(pt.*pt,2);
YY = sum(Yt.*Yt,1);
K = bsxfun(@plus,XX,YY)-2*pt*Yt;
K=K.*(K>=0);
%K=sqrt(K.*(K>=0));
% K=K.^2;
if d==1
    K=sqrt(K).^3;
else
    if d==2
        mask = K < 1e-10; % to avoid singularity.
        K = 0.5 * K .* log(K + mask) .* (K>1e-10);
    else
        K = - sqrt(K);
    end;
end
pt=tps_n_c(pt);
new_pt=pt*affine+K*non_affine;

function Y=tps_n_c(Y1)
%input Y1 is the points in n*k dimension
%output Y is the concatennated version of Y1
[m n]=size(Y1);
Y=[ones(m,1) Y1];

function [Pm,Q1,Q2,R] = tps_set_matrices(ctrl_pts)
[m,d] = size(ctrl_pts);
Pm = [ones(m,1) ctrl_pts];
[q,r]   = qr(Pm);
Q1      = q(:, 1:d+1);
Q2      = q(:, d+2:m);
R       = r(1:d+1,1:d+1);

function [idx, tri] = nearestneighbour(varargin)
%NEARESTNEIGHBOUR    find nearest neighbours
%   IDX = NEARESTNEIGHBOUR(X) finds the nearest neighbour by Euclidean
%   distance to each point (column) in X from X. X is a matrix with points
%   as columns. IDX is a vector of indices into X, such that X(:, IDX) are
%   the nearest neighbours to X. e.g. the nearest neighbour to X(:, 2) is
%   X(:, IDX(2))
%
%   IDX = NEARESTNEIGHBOUR(P, X) finds the nearest neighbour by Euclidean
%   distance to each point in P from X. P and X are both matrices with the
%   same number of rows, and points are the columns of the matrices. Output
%   is a vector of indices into X such that X(:, IDX) are the nearest
%   neighbours to P
%
%   IDX = NEARESTNEIGHBOUR(I, X) where I is a logical vector or vector of
%   indices, and X has at least two rows, finds the nearest neighbour in X
%   to each of the points X(:, I).
%   I must be a row vector to distinguish it from a single point.
%   If X has only one row, the first input is treated as a set of 1D points
%   rather than a vector of indices
%
%   IDX = NEARESTNEIGHBOUR(..., Property, Value)
%   Calls NEARESTNEIGHBOUR with the indicated parameters set. Property
%   names can be supplied as just the first letters of the property name if
%   this is unambiguous, e.g. NEARESTNEIGHBOUR(..., 'num', 5) is equivalent
%   to NEARESTNEIGHBOUR(..., 'NumberOfNeighbours', 5). Properties are case
%   insensitive, and are as follows:
%      Property:                         Value:
%      ---------                         ------
%         NumberOfNeighbours             natural number, default 1
%            NEARESTNEIGHBOUR(..., 'NumberOfNeighbours', K) finds the closest
%            K points in ascending order to each point, rather than the
%            closest point. If Radius is specified and there are not
%            sufficient numbers, fewer than K neighbours may be returned
%
%         Radius                         positive, default +inf
%            NEARESTNEIGHBOUR(..., 'Radius', R) finds neighbours within
%            radius R. If NumberOfNeighbours is not set, it will find all
%            neighbours within R, otherwise it will find at most
%            NumberOfNeighbours. The IDX matrix is padded with zeros if not
%            all points have the same number of neighbours returned. Note
%            that specifying a radius means that the Delaunay method will
%            not be used.
%
%         DelaunayMode                   {'on', 'off', |'auto'|}
%            DelaunayMode being set to 'on' means NEARESTNEIGHBOUR uses the
%            a Delaunay triangulation with dsearchn to find the points, if
%            possible. Setting it to 'auto' means NEARESTNEIGHBOUR decides
%            whether to use the triangulation, based on efficiency. Note
%            that the Delaunay triangulation will not be used if a radius
%            is specified.
%
%         Triangulation                  Valid triangulation produced by
%                                        delaunay or delaunayn
%            If a triangulation is supplied, NEARESTNEIGHBOUR will attempt
%            to use it (in conjunction with dsearchn) to find the
%            neighbours.
%
%   [IDX, TRI] = NEARESTNEIGHBOUR( ... )
%   If the Delaunay Triangulation is used, TRI is the triangulation of X'.
%   Otherwise, TRI is an empty matrix
%
%   Example:
%
%     % Find the nearest neighbour in X to each column of X
%     x = rand(2, 10);
%     idx = nearestneighbour(x);
%
%     % Find the nearest neighbours to each point in p
%     p = rand(2, 5);
%     x = rand(2, 20);
%     idx = nearestneighbour(p, x)
%
%     % Find the five nearest neighbours to points x(:, [1 6 20]) in x
%     x = rand(4, 1000)
%     idx = nearestneighbour([1 6 20], x, 'NumberOfNeighbours', 5)
%
%     % Find all neighbours within radius of 0.1 of the points in p
%     p = rand(2, 10);
%     x = rand(2, 100);
%     idx = nearestneighbour(p, x, 'r', 0.1)
%
%     % Find at most 10 nearest neighbours to point p from x within a
%     % radius of 0.2
%     p = rand(1, 2);
%     x = rand(2, 30);
%     idx = nearestneighbour(p, x, 'n', 10, 'r', 0.2)
%
%
%   See also DELAUNAYN, DSEARCHN, TSEARCH

%TODO    Allow other metrics than Euclidean distance
%TODO    Implement the Delaunay mode for multiple neighbours

% Copyright 2006 Richard Brown. This code may be freely used and
% distributed, so long as it maintains this copyright line
error(nargchk(1, Inf, nargin, 'struct'));

% Default parameters
userParams.NumberOfNeighbours = []    ; % Finds one
userParams.DelaunayMode       = 'auto'; % {'on', 'off', |'auto'|}
userParams.Triangulation      = []    ;
userParams.Radius             = inf   ;

% Parse inputs
[P, X, fIndexed, userParams] = parseinputs(userParams, varargin{:});

% Special case uses Delaunay triangulation for speed.

% Determine whether to use Delaunay - set fDelaunay true or false
nX  = size(X, 2);
nP  = size(P, 2);
dim = size(X, 1);

switch lower(userParams.DelaunayMode)
    case 'on'
        %TODO Delaunay can't currently be used for finding more than one
        %neighbour
        fDelaunay = userParams.NumberOfNeighbours == 1 && ...
            size(X, 2) > size(X, 1)                    && ...
            ~fIndexed                                  && ...
            userParams.Radius == inf;
    case 'off'
        fDelaunay = false;
    case 'auto'
        fDelaunay = userParams.NumberOfNeighbours == 1 && ...
            ~fIndexed                                  && ...
            size(X, 2) > size(X, 1)                    && ...
            userParams.Radius == inf                   && ...
            ( ~isempty(userParams.Triangulation) || delaunaytest(nX, nP, dim) );
end

% Try doing Delaunay, if fDelaunay.
fDone = false;
if fDelaunay
    tri = userParams.Triangulation;
    if isempty(tri)
        try
            tri   = delaunayn(X');
        catch
            msgId = 'NearestNeighbour:DelaunayFail';
            msg = ['Unable to compute delaunay triangulation, not using it. ',...
                'Set the DelaunayMode parameter to ''off'''];
            warning(msgId, msg);
        end
    end
    if ~isempty(tri)
        try
            idx = dsearchn(X', tri, P')';
            fDone = true;
        catch
            warning('NearestNeighbour:DSearchFail', ...
                'dsearchn failed on triangulation, not using Delaunay');
        end
    end
else % if fDelaunay
    tri = [];
end

% If it didn't use Delaunay triangulation, find the neighbours directly by
% finding minimum distances
if ~fDone
    idx = zeros(userParams.NumberOfNeighbours, size(P, 2));

    % Loop through the set of points P, finding the neighbours
    Y = zeros(size(X));
    for iPoint = 1:size(P, 2)
        x = P(:, iPoint);

        % This is the faster than using repmat based techniques such as
        % Y = X - repmat(x, 1, size(X, 2))
        for i = 1:size(Y, 1)
            Y(i, :) = X(i, :) - x(i);
        end

        % Find the closest points, and remove matches beneath a radius
        dSq = sum(abs(Y).^2, 1);
        iRad = find(dSq < userParams.Radius^2);
        if ~fIndexed
            iSorted = iRad(minn(dSq(iRad), userParams.NumberOfNeighbours));
        else
            iSorted = iRad(minn(dSq(iRad), userParams.NumberOfNeighbours + 1));
            iSorted = iSorted(2:end);
        end

        % Remove any bad ones
        idx(1:length(iSorted), iPoint) = iSorted';
    end
    %while ~isempty(idx) && isequal(idx(end, :), zeros(1, size(idx, 2)))
    %    idx(end, :) = [];
    %end
    idx( all(idx == 0, 2), :) = [];
end % if ~fDone
if isvector(idx)
    idx = idx(:)';
end % nearestneighbour




%DELAUNAYTEST   Work out whether the combination of dimensions makes
%fastest to use a Delaunay triangulation in conjunction with dsearchn.
%These parameters have been determined empirically on a Pentium M 1.6G /
%WinXP / 512MB / Matlab R14SP3 platform. Their precision is not
%particularly important
function tf = delaunaytest(nx, np, dim)
switch dim
    case 2
        tf = np > min(1.5 * nx, 400);
    case 3
        tf = np > min(4 * nx  , 1200);
    case 4
        tf = np > min(40 * nx , 5000);

        % if the dimension is higher than 4, it is almost invariably better not
        % to try to use the Delaunay triangulation
    otherwise
        tf = false;
end % switch
% delaunaytest




%MINN   find the n most negative elements in x, and return their indices
%  in ascending order
function I = minn(x, n)

% Make sure n is no larger than length(x)
n = min(n, length(x));

% Sort the first n
[xsn, I] = sort(x(1:n));

% Go through the rest of the entries, and insert them into the sorted block
% if they are negative enough
for i = (n+1):length(x)
    j = n;
    while j > 0 && x(i) < xsn(j)
        j = j - 1;
    end

    if j < n
        % x(i) should go into the (j+1) position
        xsn = [xsn(1:j), x(i), xsn((j+1):(n-1))];
        I   = [I(1:j), i, I((j+1):(n-1))];
    end
end

 %minn


%PARSEINPUTS    Support function for nearestneighbour
function [P, X, fIndexed, userParams] = parseinputs(userParams, varargin)
if length(varargin) == 1 || ~isnumeric(varargin{2})
    P           = varargin{1};
    X           = varargin{1};
    fIndexed    = true;
    varargin(1) = [];
else
    P             = varargin{1};
    X             = varargin{2};
    varargin(1:2) = [];

    % Check the dimensions of X and P
    if size(X, 1) ~= 1
        % Check to see whether P is in fact a vector of indices
        if size(P, 1) == 1
            try
                P = X(:, P);
            catch
                error('NearestNeighbour:InvalidIndexVector', ...
                    'Unable to index matrix using index vector');
            end
            fIndexed = true;
        else
            fIndexed = false;
        end % if size(P, 1) == 1
    else % if size(X, 1) ~= 1
        fIndexed = false;
    end

    if ~fIndexed && size(P, 1) ~= size(X, 1)
        error('NearestNeighbour:DimensionMismatch', ...
            'No. of rows of input arrays doesn''t match');
    end
end
% Parse the Property/Value pairs
if rem(length(varargin), 2) ~= 0
    error('NearestNeighbour:propertyValueNotPair', ...
        'Additional arguments must take the form of Property/Value pairs');
end

propertyNames = {'numberofneighbours', 'delaunaymode', 'triangulation', ...
    'radius'};
while length(varargin) ~= 0
    property = varargin{1};
    value    = varargin{2};

    % If the property has been supplied in a shortened form, lengthen it
    iProperty = find(strncmpi(property, propertyNames, length(property)));
    if isempty(iProperty)
        error('NearestNeighbour:InvalidProperty', 'Invalid Property');
    elseif length(iProperty) > 1
        error('NearestNeighbour:AmbiguousProperty', ...
            'Supplied shortened property name is ambiguous');
    end
    property = propertyNames{iProperty};

    switch property
        case 'numberofneighbours'
            if rem(value, 1) ~= 0 || ...
                    value > length(X) - double(fIndexed) || ...
                    value < 1
                error('NearestNeighbour:InvalidNumberOfNeighbours', ...
                    'Number of Neighbours must be an integer, and smaller than the no. of points in X');
            end
            userParams.NumberOfNeighbours = value;

        case 'delaunaymode'
            fOn = strcmpi(value, 'on');
            if strcmpi(value, 'off')
                userParams.DelaunayMode = 'off';
            elseif fOn || strcmpi(value, 'auto')
                if userParams.NumberOfNeighbours ~= 1
                    if fOn
                        warning('NearestNeighbour:TooMuchForDelaunay', ...
                            'Delaunay Triangulation method works only for one neighbour');
                    end
                    userParams.DelaunayMode = 'off';
                elseif size(X, 2) < size(X, 1) + 1
                    if fOn
                        warning('NearestNeighbour:TooFewDelaunayPoints', ...
                            'Insufficient points to compute Delaunay triangulation');
                    end
                    userParams.DelaunayMode = 'off';

                elseif size(X, 1) == 1
                    if fOn
                        warning('NearestNeighbour:DelaunayDimensionOne', ...
                            'Cannot compute Delaunay triangulation for 1D input');
                    end
                    userParams.DelaunayMode = 'off';
                else
                    userParams.DelaunayMode = value;
                end
            else
                warning('NearestNeighbour:InvalidOption', ...
                    'Invalid Option');
            end % if strcmpi(value, 'off')

        case 'radius'
            if isscalar(value) && isnumeric(value) && isreal(value) && value > 0
                userParams.Radius = value;
                if isempty(userParams.NumberOfNeighbours)
                    userParams.NumberOfNeighbours = size(X, 2) - double(fIndexed);
                end
            else
                error('NearestNeighbour:InvalidRadius', ...
                    'Radius must be a positive real number');
            end
    

        case 'triangulation'
            if isnumeric(value) && size(value, 2) == size(X, 1) + 1 && ...
                    all(ismember(1:size(X, 2), value))
                userParams.Triangulation = value;
            else
                error('NearestNeighbour:InvalidTriangulation', ...
                    'Triangulation not a valid Delaunay Triangulation');
            end
    end % switch property

    varargin(1:2) = [];
end % while
if isempty(userParams.NumberOfNeighbours)
    userParams.NumberOfNeighbours = 1;
end

